//***********************************************************************************************
//
//    Copyright (c) 2005 Molecular Devices.
//    All rights reserved.
//
//***********************************************************************************************
// MODULE:  ChannelStats.CPP
// PURPOSE: 
// AUTHOR:  BHI  Feb 2005
//

#include "stdafx.h"
#include "ChannelStats.hpp"

//===============================================================================================
// FUNCTION: Constructor
// PURPOSE:  Object initialization.
//
CChannelStats::CChannelStats()
{
   MEMBERASSERT();
   m_uInx    = 0;
   m_uSeqLen = 0;
}

//===============================================================================================
// FUNCTION: Destructor
// PURPOSE:  Object cleanup.
//
CChannelStats::~CChannelStats()
{
   MEMBERASSERT();
}

//===============================================================================================
// FUNCTION: SetBuffer
// PURPOSE:  .
//
void CChannelStats::SetBuffer(MDC_DATABUFFER *pBuffer, UINT uInx, UINT uSeqLen)
{
   MEMBERASSERT();
   m_Next.Set(pBuffer);
   m_uInx    = uInx;
   m_uSeqLen = uSeqLen;
   Clear();
}

//===============================================================================================
// FUNCTION: NewData
// PURPOSE:  .
//
void CChannelStats::NewData(UINT uSeq)
{
   MEMBERASSERT();

   UINT uRemaining = uSeq * m_uSeqLen;
   while (uRemaining)
   {
      UINT uLeft   = m_Next.Remaining();
      UINT uCount  = min(uRemaining, uLeft);
      short *pNext = m_Next.GetPtr() + m_uInx;

      for (UINT i=0; i<uCount; i+=m_uSeqLen)
      {
         m_StdDev.Add(pNext[i]);
         m_MinMax.Add(pNext[i]);
      }

      m_Next     += uCount;
      uRemaining -= uCount;
   }
}

//===============================================================================================
// FUNCTION: GetResults
// PURPOSE:  .
//
void CChannelStats::GetResults(ChannelStats &Stats)
{
   MEMBERASSERT();
   Stats = ChannelStats();

   Stats.nMin     = m_MinMax.GetMin();
   Stats.nMax     = m_MinMax.GetMax();
   Stats.dMean    = m_StdDev.GetMean();
   Stats.dStdDev  = m_StdDev.GetStdDev();
   Stats.uCount   = m_StdDev.GetCount();
}

//===============================================================================================
// FUNCTION: Clear
// PURPOSE:  .
//
void CChannelStats::Clear()
{
   MEMBERASSERT();
   m_StdDev.Clear();
   m_MinMax.Clear();
}
