// DD1550Dlg.h : header file
//

#pragma once

#include "\AxonDev\Comp\WinScope\WinScope.h"
#include "\AxonDev\Comp\DriversUSB2\USB2Driver\WD_Control.hpp"
#include "\AxonDev\Comp\CommonLib\DataBuffer.hpp"
#include "\AxonDev\Comp\CommonLib\DataIterator.hpp"
#include "\AxonDev\Comp\CommonLib\AcqTimer.hpp"
#include "Protocol.hpp"
#include "ChannelStats.hpp"
#include "TestInputsDlg.hpp"
#include "TestDlg.h"
#include "afxwin.h"
#include <vector>

#define CDD1550BDlg_BASE CDialog

// CDD1550Dlg dialog
class CDD1550BDlg : public CDialog
{
	// Implementation
private:
	enum { IDD = IDD_DD1550B_DIALOG };
	CButton	             m_Start;
	CButton	             m_EditProtocol;
	CButton	             m_Copy;
	CButton	             m_About;
	CAnimateCtrl	       m_Animation;

	UINT                  m_uSerial;
	UINT                  m_uFirstChannel;
	HICON                 m_hIcon;

	SC_TIMEBASE           m_Timebase;
	CArrayPtrEx<SC_TRACE> m_Traces;
	MDC_DataBufferList       m_AIBuffers;
	MDC_DataBufferList       m_AOBuffers;
	MDC_DataBufferList       m_OutPeriod;
	MDC_DataIterator         m_NextIn;
	MDC_DataIterator         m_NextOut;
	MDC_DataIterator         m_NextWave;
	bool                  m_bEvent;
	std::vector<SC_SYNCH> m_Synch;
	HWND                  m_hwndScope;

	DD1550B_Device        *m_pDevice;
	DD1550B_Info           m_DeviceInfo;
	DD1550B_Calibration    m_CalData;
	Protocol              m_Protocol;
	TestSummary           m_Summary;
	UINT                  m_uAISeqLen;
	UINT                  m_uAOSeqLen;
	UINT                  m_uTimerID;

	CAcqTimer             m_AcqTimer;
	bool                  m_bInit;
	UINT                  m_uLastTime;
	LONGLONG              m_uAISequences;
	LONGLONG              m_uAOSequences;

	CChannelStats         m_Stats;

	// Modeless dialogs
	CTestInputsDlg        m_TestInputsDlg;

	// Implementation
protected:

	// Generated message map functions
	virtual BOOL OnInitDialog();
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
	virtual void OnOK() {}

	afx_msg void OnDestroy();
	afx_msg void OnTimer(UINT_PTR uIDEvent);
	afx_msg void OnAbout();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnClear();
	afx_msg LRESULT OnOpenDevice(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnScopeMessage(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnUIUpdate(WPARAM wParam, LPARAM lParam);
	afx_msg void OnCopy();
	afx_msg void OnStart();
	afx_msg void OnBnClickedProtocol();
	afx_msg void OnBnClickedOutputs();
	afx_msg void OnBnClickedRotate();
	afx_msg void OnBnClickedTestInputs();
	afx_msg void OnBnClickedCalibration();
	afx_msg void OnBnClickedDO(UINT uID);
	DECLARE_MESSAGE_MAP()

private:
	bool OutOfMemory()   { MessageBox("Out of memory!"); return false; }
	bool ShowDeviceError(int nError=0);
	void BuildOutputWave();
	bool SetupScope();
	bool OpenDevice(UINT uSerial, UINT uFirstChannel);
	void CloseDevice();
	void UIUpdate();
	bool Start();
	void Stop(bool bShutdown=false);
	void ShowPeriod(double dUS);
	void SetPeriod(double dPeriodUS);
	void FixLayout();
	bool SetProtocol();
	void EnableControlPanel();
	void ReadInputs();
	void DetectEvents(UINT uSequences);
	void TopUpOutput(UINT uNewSeq);
	void ShowSuccess(UINT uID, bool bSuccess);

	void FillBuffer(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const;
	void FillSineWave(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const;
	void FillSawTooth(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const;
	void FillZero(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const;
	void FillSquareWave(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const;

	// Construction
public:
	CDD1550BDlg(CWnd* pParent = NULL);	// standard constructor
	~CDD1550BDlg();

	void SetDevice(UINT uSerial, UINT uFirstChannel)
	{
		m_uSerial       = uSerial; 
		m_uFirstChannel = uFirstChannel;
	}
	afx_msg void OnBnClickedOpenDevice();
	afx_msg void OnBnClickedTestTelegraphs();
	afx_msg void OnBnClickedTestDigitalOUTs();
	afx_msg void OnBnClickedTestAiNoise();
	afx_msg void OnBnClickedTestReport();
	afx_msg void OnBnClickedTestAiCrosstalk();
	afx_msg void OnBnClickedTestAoNoise();
	afx_msg void OnBnClickedChkCal();
};



/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog

class CAboutDlg : public CDialog
{
private:
	enum { IDD = IDD_ABOUT };
	DD1550B_Info    m_Info;
	DD1550B_Device *m_pDevice;
	CFont          m_Font;
	CStatic	      m_Product;
	CStatic        m_USB1;
	bool           m_bSerialChanged;

	// ClassWizard generated virtual function overrides
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual BOOL OnInitDialog();

	DECLARE_MESSAGE_MAP()

	// Construction
public:
	CAboutDlg(CWnd* pParent, DD1550B_Device *pDevice);   // standard constructor
	afx_msg void OnBnClickedSetSerial();

	bool SerialChanged() const { return m_bSerialChanged; }
};
