#pragma once

struct TestSummary
{
	bool   bCalibration;
	bool   bCalibrationChk;
	bool   bTelegraphs;
	bool   bDigitalOuts;
	bool   bAINoise;
	bool   bAICrosstalk;
	bool   bAONoise;

	// Telegraph BNC tests
	bool   abTelegraphs[4];
	bool   bTag;
	bool   bStart;

	// Digital outputs test
	bool   abDigitalOuts[16];			//NST change to support 16 DO from 8  req 2
	bool   bScopeOut;

	// Input Crosstalk
	bool   abCrosstalk[16];
	double adNoiseRMS[16];
	double adXTalkDB[16];

	// Input noise
	bool   abAINoise[8]; // NST change to Support 8 AI  from 16 req 1
	double adAINoise[8]; // NST change to Support 8 AI from 16 req 1


	// Output noise
	bool   abAONoise[8];//  NST change to Support 8 AO from 4 req 1
	double adAONoise[8];//  NST change to Support 8 AO from 4 req 1

	TestSummary()
	{
		memset(this, 0, sizeof(*this));
	}
};

// CDigitalOutputsDlg dialog

class CTestDlg : public CDialog
{
protected:
	DD1550B_Device       *m_pDevice;
	DD1550B_Info          m_Info;
	DD1550B_Calibration   m_CalData;
	TestSummary         *m_pSummary;

	bool                 m_bSuccess;
	char                 m_szLastError[256];
	int                  m_nLastError;

protected:
	bool CHK(bool bRval)
	{
		if (bRval)
			return true;

		m_nLastError = m_pDevice->GetLastError();
		m_pDevice->GetLastErrorText(m_szLastError, ELEMENTS_IN(m_szLastError));
		ERRORMSG2("DD1550B ERROR (%d): %s", m_nLastError, m_szLastError);
		return false;
	}

	void SineWave(UINT uDAC, MDC_DATABUFFER *pDB, UINT uOffset, UINT uSkip, 
		double dSeqIntervalUS, double dFreqHz, double dRangeV, double dOffsetV)
	{
		double dPeriod = 1 / dFreqHz;

		// Must be at least a whole period...
		ASSERT(pDB->uNumSamples >= dPeriod / dSeqIntervalUS * 1E6);

		ADC_VALUE *pn = pDB->pnData + uOffset;
		const double c_d2Pi = 8 * atan(1.0);

		for (UINT i=0; i<pDB->uNumSamples; i+=uSkip)
		{
			double dTime    = i * dSeqIntervalUS / 1E6;
			double dRadians = dTime / dPeriod * c_d2Pi;
			double dVolts   = dRangeV/2 * sin(dRadians) + dOffsetV;

			pn[i] = DD1550B_VoltsToDAC(m_CalData, uDAC, dVolts);
		}
	}

	bool SetAOValue(UINT uDAC, double dVolts)
	{
		return CHK(m_pDevice->SetAOValue(uDAC, DD1550B_VoltsToDAC(m_CalData, uDAC, dVolts)));
	}

	void SetMsg(LPCSTR psz)
	{
		SetDlgItemText(IDC_MSG, psz);
	}

public:
	CTestDlg(UINT uID, CWnd* pParent, DD1550B_Device *pDevice, TestSummary *pSummary)
		: CDialog(uID, pParent)
	{
		m_pDevice        = pDevice;
		m_pSummary       = pSummary;
		m_bSuccess       = false;
		m_nLastError     = 0;
		m_szLastError[0] = '\0';

		
		VERIFY(m_pDevice->GetDeviceInfo(&m_Info));
		VERIFY(m_pDevice->GetCalibrationParams(m_CalData));
	}

	bool IsSuccess() const { return m_bSuccess; }
};

inline double Decibels(double dNumerator, double dDenominator)
{
	return 20 * log10(max(dNumerator, 1.0) / max(dDenominator, 1E-12));
}
