// TestInputsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DD1550B.h"
#include "TestInputsDlg.hpp"
#include ".\testinputsdlg.hpp"


// CTestInputsDlg dialog

IMPLEMENT_DYNAMIC(CTestInputsDlg, CDialog)

//===============================================================================================
// FUNCTION: Constructor
// PURPOSE:  Object initialization.
//
CTestInputsDlg::CTestInputsDlg()
: CDialog(CTestInputsDlg::IDD)
{
	MEMBERASSERT();
	m_pDevice   = NULL;
	m_pProtocol = NULL;
	memset(m_abAI, 0, sizeof(m_abAI));
	m_bDI = false;
}

//===============================================================================================
// FUNCTION: Update
// PURPOSE:  .
//
void CTestInputsDlg::Update()
{
	MEMBERASSERT();

	memset(m_abAI, 0, sizeof(m_abAI));
	for (UINT i=0; i<m_pProtocol->uAIChannels; i++)
		m_abAI[m_pProtocol->anAIChannels[i]] = true;
	m_bDI = m_pProtocol->bDIEnable;
	UpdateData(false);

	GetDlgItem(IDC_READ_INPUTS)->EnableWindow(!m_pDevice->IsAcquiring());
}

void CTestInputsDlg::SetDevice(DD1550B_Device *pDevice, const DD1550B_Protocol *pProtocol)
{
	MEMBERASSERT();

	m_pDevice   = pDevice;
	m_pProtocol = pProtocol;
	Update();
}

//===============================================================================================
// FUNCTION: ShowDeviceError
// PURPOSE:  .
//
bool CTestInputsDlg::ShowDeviceError(int nError)
{
	if (!nError && m_pDevice)
		nError = m_pDevice->GetLastError();
	if (!nError)
		return false;

	char sz[256] = "";
	DD1550B_GetErrorText(nError, sz, ELEMENTS_IN(sz));
	MessageBox(sz, "Error");
	return false;
}

void CTestInputsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	DDX_Check(pDX, IDC_DI, m_bDI);
	for (UINT i=0; i<ELEMENTS_IN(m_abAI); i++)
		DDX_Check(pDX, IDC_AI0+i, m_abAI[i]);

	if (m_pDevice && !pDX->m_bSaveAndValidate)
		GetDlgItem(IDC_READ_INPUTS)->EnableWindow(!m_pDevice->IsAcquiring());
}

BEGIN_MESSAGE_MAP(CTestInputsDlg, CDialog)
	ON_BN_CLICKED(IDC_TELEGRAPHS, OnBnClickedTelegraphs)
	ON_BN_CLICKED(IDC_READ_INPUTS, OnBnClickedReadInputs)
	ON_BN_CLICKED(IDC_SETALL, OnBnClickedSetall)
	ON_BN_CLICKED(IDC_CLEARALL, OnBnClickedClear)
END_MESSAGE_MAP()

// CTestInputsDlg message handlers

void CTestInputsDlg::OnBnClickedReadInputs()
{
	UpdateData();

	if(m_pDevice== NULL)  // NST 
	{	
		ShowDeviceError(DD1550B_ERROR_NOTPRESENT);
		return ;
	}

	char szInputs[1024] = "", sz[64] = "";
	for (UINT i=0; i<ELEMENTS_IN(m_abAI); i++)
	{
		if (!m_abAI[i])
			continue;

		short nValue = 0;
		if (!m_pDevice->GetAIValue(i, nValue))
		{
			ShowDeviceError();
			break;
		}

		double dVolts = nValue * 10.0 /  32767;  // NST 29.10.12 Change from 32768 to 32767
		MDC_sprintf(sz, ELEMENTS_IN(sz), "AI%d = %d (%g V)\n", i, nValue, dVolts);
		MDC_strncatz(szInputs, sz);
	}

	if (m_bDI)
	{
		WORD wValue = 0;
		if (!m_pDevice->GetDIValue(wValue))
			ShowDeviceError();

		MDC_sprintf(sz, ELEMENTS_IN(sz), "DI = 0x%02X\n", wValue);
		MDC_strncatz(szInputs, sz);
	}
	SetDlgItemText(IDC_STATUS, szInputs);
}

void CTestInputsDlg::OnBnClickedTelegraphs()
{
	UpdateData();
   
	if(m_pDevice== NULL)  // NST 
	{	
		ShowDeviceError(DD1550B_ERROR_NOTPRESENT);
		return ;
	}

	ADC_VALUE anTele[4] = { 0 };
	if (!m_pDevice->ReadTelegraphs(0, anTele, ELEMENTS_IN(anTele)))
		ShowDeviceError();

	char szInputs[1024] = "", sz[64] = "";
	for (UINT i=0; i<ELEMENTS_IN(anTele); i++)
	{
		double dVolts = anTele[i] * 10.0 / 32720;  // NST 6.9.12 Change from 32768 to 32720
		MDC_sprintf(sz, ELEMENTS_IN(sz), "T%d = %d (%g V)\n", i, anTele[i], dVolts);
		MDC_strncatz(szInputs, sz);
	}
	SetDlgItemText(IDC_STATUS, szInputs);
}

void CTestInputsDlg::OnBnClickedSetall()
{
	for (UINT i=0; i<DD1550B_MAX_AI_CHANNELS; i++)
		CheckDlgButton(IDC_AI0+i, true);

	CheckDlgButton(IDC_DI, true);
}

void CTestInputsDlg::OnBnClickedClear()
{
	for (UINT i=0; i<DD1550B_MAX_AI_CHANNELS; i++)
		CheckDlgButton(IDC_AI0+i, false);

	CheckDlgButton(IDC_DI, false);
}

void CTestInputsDlg::OnCancel()
{
	ShowWindow(SW_HIDE);
	GetParent()->PostMessage(WM_UIUPDATE);
}
