// DD1550Dlg.cpp : implementation file
//

#include "stdafx.h"
#include "\AxonDev\Comp\CommonLib\MDCDefn.h"
#include "\AxonDev\Comp\CommonLib\MDCDebug.h"
#include "\AxonDev\Comp\CommonLib\Colors.h"
#include "\AxonDev\Comp\CommonLib\WaitCursor.hpp"
#include "\AxonDev\Comp\CommonLib\Version.hpp"
#include "\AxonDev\Comp\CommonLib\FileIO.hpp"
#include "\AxonDev\Comp\CommonLib\FilePath.h"
//#include "\AxonDev\Comp\AxoUtils32\AxoUtils32.h"
#include "\AxonDev\Comp\CommonCtls\CommonCtls.h"

// DD1550B hardware constants.
#include "\Hardware\AxUSB2\DD1550B\DD1550B.h"

#include <math.h>

#include "DD1550B.h"
#include "DD1550BDlg.h"
#include "ChooseDeviceDlg.hpp"


#include "ProtocolDlg.hpp"

#include ".\dd1550Bdlg.h"
//#include <iomanip>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

const UINT c_uTimerID     = 666;
const UINT c_uTimerPeriod = 100;

const UINT WM_SCOPE_NOTIFY = RegisterWindowMessage(SC_MESSAGE_STR);

const float c_fUUMax =  10.2F;
const float c_fUUMin = -10.2F;

const float c_fADCtoUUFactor = 10.0F/32768.0F;  ; 
const float c_fADCtoUUShift  = 0;

//===============================================================================================
// FUNCTION: Constructor
// PURPOSE:  .
//
CDD1550BDlg::CDD1550BDlg(CWnd* pParent /*=NULL*/)
: CDialog(CDD1550BDlg::IDD, pParent)
{
	m_hIcon         = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_uSerial       = DD1550B_ANY_DEVICE;
	m_uFirstChannel = 0;
	m_pDevice       = NULL;
	m_bInit         = false;
	m_uLastTime     = 0;
	m_uAISequences  = 0;
	m_uAOSequences  = 0;
	m_uAISeqLen     = 0;
	m_uAOSeqLen     = 0;
	m_hwndScope     = NULL;
	m_bEvent        = false;

	m_Protocol.Load();

	MCC_Initialize(AfxGetInstanceHandle());
}

//===============================================================================================
// FUNCTION: Destructor
// PURPOSE:  .
//
CDD1550BDlg::~CDD1550BDlg()
{
	m_Protocol.Save();
	MCC_Shutdown();
}

//===============================================================================================
// FUNCTION: DoDataExchange
// PURPOSE:  .
//
void CDD1550BDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_START, m_Start);
	DDX_Control(pDX, IDC_PROTOCOL, m_EditProtocol);
	DDX_Control(pDX, IDC_COPY, m_Copy);
	DDX_Control(pDX, IDC_ABOUT, m_About);
	DDX_Control(pDX, IDC_ANIMATION, m_Animation);
	

	for (UINT i=0; i<ELEMENTS_IN(m_Protocol.adAOHolding); i++)
	{
		
		DDX_TextFormat(pDX, IDC_AO0+i, m_Protocol.adAOHolding[i], 4);
		DDV_MinMaxDouble(pDX, m_Protocol.adAOHolding[i], -10.0, 10.0);
		((CEdit *)GetDlgItem(IDC_AO0+i))->SetLimitText(8);
		
		
	}
	for (UINT i=0; i<DD1550B_MAX_DO_CHANNELS -1; i++)
		DDX_CheckBitFlag(pDX, IDC_DO0+i, m_Protocol.wDOHolding, WORD(1<<i)); // Check Bit Flag for DO0 to DO14
    
	
	if( ((m_Protocol.uFlags) & DD1550B_FLAG_SCOPE_OUT) !=0)    // NST update DO 15 status according to digital scope output status  req 2
		{
			DDX_CheckBitFlag(pDX, IDC_SCOPE, m_Protocol.wDOHolding, WORD(1<<(DD1550B_MAX_DO_CHANNELS-1)));     
			GetDlgItem(IDC_DO15)->EnableWindow(0);
		}
		else
		{	DDX_CheckBitFlag(pDX, IDC_DO15, m_Protocol.wDOHolding, WORD(1<<(DD1550B_MAX_DO_CHANNELS-1)));    
			GetDlgItem(IDC_DO15)->EnableWindow(1);
		}


}

//===============================================================================================
// MFC MESSAGE MAP
//
BEGIN_MESSAGE_MAP(CDD1550BDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_ABOUT, OnAbout)
	ON_WM_SIZE()
	ON_REGISTERED_MESSAGE(WM_SCOPE_NOTIFY, OnScopeMessage)
	ON_MESSAGE(WM_UIUPDATE, OnUIUpdate)
	ON_MESSAGE(WM_OPEN_DEVICE, OnOpenDevice)
	ON_BN_CLICKED(IDC_COPY, OnCopy)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDC_START, OnStart)
	ON_BN_CLICKED(IDC_PROTOCOL, OnBnClickedProtocol)
	ON_BN_CLICKED(IDC_OUTPUTS, OnBnClickedOutputs)
	ON_BN_CLICKED(IDC_ROTATE, OnBnClickedRotate)
	ON_BN_CLICKED(IDC_TEST_INPUTS, OnBnClickedTestInputs)
	ON_BN_CLICKED(IDC_OPEN_DEVICE, OnBnClickedOpenDevice)
	ON_CONTROL_RANGE(BN_CLICKED, IDC_DO0, IDC_DO15, OnBnClickedDO)
END_MESSAGE_MAP()

//===============================================================================================
// FUNCTION: OnInitDialog
// PURPOSE:  .
//
BOOL CDD1550BDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	CWaitCursor Wait;

	m_uLastTime    = 0;
	m_uAISequences = 0;
	m_uAOSequences = 0;

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_Animation.Open(IDR_BUSY);

	m_Copy.ModifyStyle(0, BS_ICON);
	m_Copy.SetIcon(AfxGetApp()->LoadIcon(IDI_COPY));

	m_Start.ModifyStyle(0, BS_ICON);
	m_Start.SetIcon(AfxGetApp()->LoadIcon(IDI_START));

	m_EditProtocol.ModifyStyle(0, BS_ICON);
	m_EditProtocol.SetIcon(AfxGetApp()->LoadIcon(IDI_PROTOCOL));

	m_About.ModifyStyle(0, BS_ICON);
	m_About.SetIcon(::LoadIcon(NULL, IDI_QUESTION));

	SetDlgItemText(IDC_PERIOD, "");
	SetDlgItemText(IDC_STATUS, "");
	SetDlgItemText(IDC_SERIAL, "");

	SCOPE_CheckAPIVersion(SC_APIVERSION_STR);
	m_hwndScope = ::GetDlgItem(m_hWnd, IDC_SCOPE);
	FixLayout();

	m_TestInputsDlg.Create(CTestInputsDlg::IDD, this);
	return TRUE;  // return TRUE  unless you set the focus to a control
}

//===============================================================================================
// FUNCTION: ShowDeviceError
// PURPOSE:  .
//
bool CDD1550BDlg::ShowDeviceError(int nError)
{
	if (!nError && m_pDevice)
		nError = m_pDevice->GetLastError();
	if (!nError)
		return false;

	char sz[256] = "";
	DD1550B_GetErrorText(nError, sz, ELEMENTS_IN(sz));
	MessageBox(sz, "Error");
	return false;
}

//===============================================================================================
// CLASS:   CRunAnimation
// PURPOSE: .
//
class CRunAnimation
{
private:
	CWaitCursor   m_Wait;
	CAnimateCtrl *m_pA;
	CWnd         *m_pT;

public:
	CRunAnimation(CAnimateCtrl *pA, CWnd *pT)
	{
		m_pA = pA;
		m_pT = pT;
		m_pT->ShowWindow(SW_SHOWNORMAL);
		m_pA->ShowWindow(SW_SHOWNORMAL);
		m_pA->Play(0, UINT(-1), UINT(-1));
	}
	~CRunAnimation()
	{
		m_pA->ShowWindow(SW_HIDE);
		m_pT->ShowWindow(SW_HIDE);
		m_pA->Stop();
		m_pA->Seek(0);
	}
};

static void MaskInBit(WORD *pwData, UINT uSeq, UINT uSeqLen, WORD wBit)
{
	for (UINT i=0; i<uSeq; i++)
		pwData[i*uSeqLen] |= wBit;
}

void CDD1550BDlg::FillSineWave(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const
{
	const double dMax       = m_Protocol.dOutMaxV;
	const double dMin       = m_Protocol.dOutMinV;
	const double d2Pi       = 2 * 3.1415926535;
	const double dCenter    = (dMax+dMin)/2;
	const double dHalfHeight= (dMax-dMin)/2;

	for (UINT uNext=0; uNext < uSequences; uNext++)
		pnData[uNext*uSeqLen] = DD1550B_VoltsToDAC(m_CalData, uDAC, dHalfHeight * sin(double(uNext) / uSequences * d2Pi) + dCenter);
}

void CDD1550BDlg::FillSawTooth(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const
{
   DAC_VALUE nMin = DD1550B_VoltsToDAC(m_CalData, uDAC, m_Protocol.dOutMinV);
   DAC_VALUE nMax = DD1550B_VoltsToDAC(m_CalData, uDAC, m_Protocol.dOutMaxV);
	for (UINT uNext=0; uNext < uSequences; uNext++)
   {
		pnData[uNext*uSeqLen] = DAC_VALUE(nMin + double(uNext) * (nMax-nMin) / uSequences);
   }
}

void CDD1550BDlg::FillZero(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const
{
	for (UINT uNext=0; uNext < uSequences; uNext++)
		pnData[uNext*uSeqLen] = 0;
}

void CDD1550BDlg::FillSquareWave(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const
{
   DAC_VALUE nMin = DD1550B_VoltsToDAC(m_CalData, uDAC, m_Protocol.dOutMinV);
   DAC_VALUE nMax = DD1550B_VoltsToDAC(m_CalData, uDAC, m_Protocol.dOutMaxV);

	UINT uNext = 0;
	for (; uNext < uSequences/2; uNext++)
		pnData[uNext*uSeqLen] = nMin;
	for (; uNext < uSequences; uNext++)
		pnData[uNext*uSeqLen] = nMax;
}

void CDD1550BDlg::FillBuffer(DAC_VALUE *pnData, UINT uSequences, UINT uDAC, UINT uSeqLen) const
{
	switch (uDAC)
	{
	case 0:
		FillSineWave(pnData, uSequences, uDAC, uSeqLen);
		break;
	case 1:
		FillSawTooth(pnData, uSequences, uDAC, uSeqLen);
		break;
	case 2:
		FillZero(pnData, uSequences, uDAC, uSeqLen);
		break;
	case 3:
		FillSquareWave(pnData, uSequences, uDAC, uSeqLen);
		break;
	}
}

//===============================================================================================
// FUNCTION: BuildOutputWave
// PURPOSE:  .
//
void CDD1550BDlg::BuildOutputWave()
{
	if (m_uAOSeqLen==0)
		return;

	// Build a single period of the output waveform
	UINT uSequences = UINT(m_Protocol.dOutPeriodMS*1E3 / m_Protocol.dSequencePeriodUS);
	uSequences      = max(uSequences, 1);
	UINT uSamples   = uSequences * m_uAOSeqLen;
	m_OutPeriod.Alloc(1, uSamples);
	m_OutPeriod.Zero();

	MDC_DATABUFFER *pOutPeriod  = m_OutPeriod;
	ADC_VALUE  *pnOutPeriod = pOutPeriod->pnData;

	for (UINT i=0; i<m_Protocol.uAOChannels; i++)
		FillBuffer(pnOutPeriod+i, uSequences, m_Protocol.anAOWaveType[m_Protocol.anAOChannels[i]], m_uAOSeqLen);

	if (m_Protocol.bDOEnable)
	{
		WORD *pwDigitalOuts = (WORD *)(pnOutPeriod + m_Protocol.uAOChannels);

		for (UINT i=0; i<DD1550B_MAX_DO_CHANNELS; i++)
		{
			switch (m_Protocol.anDOWaveType[i])
			{
			case 0:     // Prezero'd buffer do nothing.
				break;
			case 1:
				MaskInBit(pwDigitalOuts, uSequences, m_uAOSeqLen, WORD(1<<i));
				break;
			case 2:
				MaskInBit(pwDigitalOuts, uSequences/2, m_uAOSeqLen, WORD(1<<i));
				break;
			default:
				ERRORMSG0("Unexpected DO wave type.");
				break;
			}
		}
	}

	m_NextWave = m_OutPeriod;
	m_NextOut  = m_AOBuffers;

	if (m_uAOSeqLen)
		TopUpOutput(m_AOBuffers.GetTotalSamples()/m_uAOSeqLen);
}

//===============================================================================================
// FUNCTION: FillBuffer
// PURPOSE:  .
//
void CDD1550BDlg::TopUpOutput(UINT uNewSeq)
{
	if (m_uAOSeqLen==0)
		return;

	UINT uRemaining = uNewSeq * m_uAOSeqLen;
	while (uRemaining)
	{
		UINT uChunk = min(uRemaining, m_NextOut.Remaining());
		uChunk      = min(uChunk, m_NextWave.Remaining());

		memcpy(m_NextOut.GetPtr(), m_NextWave.GetPtr(), uChunk*sizeof(DAC_VALUE));
		m_NextOut  += uChunk;
		m_NextWave += uChunk;
		uRemaining -= uChunk;
	}
}

//===============================================================================================
// FUNCTION: SetupScope
// PURPOSE:  .
//
bool CDD1550BDlg::SetupScope()
{
	MEMBERASSERT();

	m_uAISeqLen  = m_Protocol.uAIChannels;
	if (m_Protocol.bDIEnable)
		m_uAISeqLen++;
	m_uAOSeqLen  = m_Protocol.uAOChannels;
	if (m_Protocol.bDOEnable)
		m_uAOSeqLen++;

	if (!m_Traces.Alloc(m_uAISeqLen))
		return false;

	UINT uAISamples = m_uAISeqLen * 1024 * 1024;
	UINT uAOSamples = m_uAOSeqLen * 1024 * 1024;

	VERIFY(m_AIBuffers.Alloc(1, uAISamples));
	m_AIBuffers.Zero();

	VERIFY(m_AOBuffers.Alloc(1, uAOSamples));
	m_AOBuffers.Zero();
	BuildOutputWave();

	SC_TRACE Trace;
	Trace.uMxOffset       = 0;                   // Multiplex offset to data in buffer
	Trace.uSkip           = m_uAISeqLen;         // Distance (in samples) between adjacent samples
	Trace.rgbColor        = RGB_BLUE;            // Pen color used to draw trace
	Trace.uPenWidth       = 1;                   // Pen width in pixels.
	Trace.bDrawPoints     = FALSE;               // TRUE = Draw disconnected points
	Trace.bFloatData      = FALSE;               // TRUE = Draw data from the float array.
	Trace.fVertProportion = 1.0F;                // Relative proportion of client area to use
	Trace.fUUTop          = c_fUUMax;            // Top of window in UserUnits
	Trace.fUUBottom       = c_fUUMin;            // Bottom of window in UserUnits
	Trace.fADCtoUUFactor  = c_fADCtoUUFactor;    // coeff to convert ADC -> UserUnits
	Trace.fADCtoUUShift   = c_fADCtoUUShift;     //  UU = ADC * fADCToUUFactor + fADCToUUShift
	Trace.fMaxValue       = c_fUUMax;            // Max extreme at Full Scale
	Trace.fMinValue       = c_fUUMin;            // Min extreme at Full Scale
	Trace.uMxOffset       = 0;
	MDC_strncpyz(Trace.szName, " ");
	MDC_strncpyz(Trace.szUnits, "V");

	for (UINT i=0; i<m_Traces.GetCount(); i++)
	{
		if (i < m_Protocol.uAIChannels)
			MDC_sprintf(Trace.szName, ELEMENTS_IN(Trace.szName), "AI%d", m_Protocol.anAIChannels[i]);
		else
		{
			MDC_strncpyz(Trace.szName, "DI");
			MDC_strncpyz(Trace.szUnits, " ");
		}
		m_Traces[i] = Trace;
		m_Traces[i].uMxOffset = i;
	}

	m_Timebase.dSequenceIntervalMS = m_Protocol.dSequencePeriodUS / 1E3;
	m_Timebase.uNumSequences       = 1000000;

	SCOPE_ChangeMode( m_hwndScope, SC_CONTINUOUS );
	SCOPE_ResetAcqCounters( m_hwndScope );
	SCOPE_SetTimebases( m_hwndScope, &m_Timebase, 1 );
	SCOPE_SetXAxisUnits( m_hwndScope, "ms");
	SCOPE_SetTraces( m_hwndScope, m_Traces, m_Traces.GetCount() );
	SCOPE_SetDataBuffers( m_hwndScope, m_AIBuffers, NULL, NULL, NULL, m_AIBuffers.GetCount() );
	SCOPE_SetEventDisplay( m_hwndScope, SC_SHOWEVENTS, 0);

	DWORD dwFlags = SC_HIDETWIRLER       | SC_HIDEPRETRIGGER   | SC_NOEDITPROTOCOL |
		SC_NOPRINTWHENFILLED | SC_NODISPLAYAVERAGE | SC_NOEDITSIGNAL;
	SCOPE_SetFlags( m_hwndScope, SC_SETFLAGS, dwFlags);
	return true;
}

//===============================================================================================
// FUNCTION: OpenDevice
// PURPOSE:  .
//
bool CDD1550BDlg::OpenDevice(UINT uSerial, UINT uFirstChannel)
{
	CRunAnimation Run(&m_Animation, GetDlgItem(IDC_MSG));

	int nError = 0;
	bool bIgnoreCRCErr = true;
	m_pDevice = DD1550B_OpenDevice(uSerial, &nError, bIgnoreCRCErr);
	if (!m_pDevice)
		return ShowDeviceError(nError);

	if (m_pDevice->GetLastError() == DD1550B_ERROR_CRC_CHECKFAIL)
		ShowDeviceError(nError);

	bool bSetDeviceInfo = m_pDevice->SetDeviceInfo(); // NST Set Device info After fpga file load
	
	if (!m_pDevice->GetDeviceInfo(&m_DeviceInfo))
	{
		CloseDevice();
		return ShowDeviceError();
	}
	VERIFY(m_pDevice->GetCalibrationParams(m_CalData)); 

	m_TestInputsDlg.SetDevice(m_pDevice, &m_Protocol);

	VERIFY(SetupScope());
	SCOPE_FullScaleXAxis( m_hwndScope );

	char sz[128] = "";
	MDC_sprintf(sz, ELEMENTS_IN(sz), "Serial\n#%d", m_DeviceInfo.uSerialNumber);
	SetDlgItemText(IDC_SERIAL, sz);

	if(m_DeviceInfo.ucMaxSamplingFreq)     // NST  update min sequnce period if Max sampling Frequncy is 250 kHz
	{
		if(m_Protocol.dSequencePeriodUS<4) // NST Min Sequence Period must be greater than 4
			m_Protocol.dSequencePeriodUS = 4;
	}
	ShowPeriod(m_Protocol.dSequencePeriodUS);

	VERIFY(SetProtocol());

	m_Summary = TestSummary();

	OnBnClickedTestInputs();
	OnBnClickedOutputs();
	return true;
}

//===============================================================================================
// FUNCTION: CloseDevice
// PURPOSE:  .
//
void CDD1550BDlg::CloseDevice()
{
	MEMBERASSERT();
	if (m_pDevice)
	{
		Stop(true);
		DD1550B_CloseDevice(m_pDevice);
		m_pDevice = NULL;
	}
}

//===============================================================================================
// FUNCTION: OnOpenDevice
// PURPOSE:  .
//
LRESULT CDD1550BDlg::OnOpenDevice(WPARAM wParam, LPARAM lParam)
{
	MEMBERASSERT();

	if (m_uSerial==DD1550B_ANY_DEVICE)
	{
		CloseDevice();
		CRunAnimation Run(&m_Animation, GetDlgItem(IDC_MSG));

		UINT uDevices = DD1550B_CountDevices();

		if (uDevices == 1)
			m_uSerial = DD1550B_ANY_DEVICE;
		else
		{
			CChooseDeviceDlg Dlg(this, max(uDevices, 4));

			if (Dlg.DoModal()==IDCANCEL)
				return 0;

			m_uSerial       = Dlg.GetSerial();
			m_uFirstChannel = Dlg.GetFirstChannel();
		}
	}
	OpenDevice(m_uSerial, m_uFirstChannel);
	return 0;
}

//===============================================================================================
// FUNCTION: Stop
// PURPOSE:  .
//
void CDD1550BDlg::Stop(bool bShutdown)
{
	if (!m_pDevice || !m_AcqTimer.IsRunning() || !m_hWnd)
		return;

	m_AcqTimer.Stop();
	m_pDevice->StopAcquisition();
	m_Start.SetIcon(AfxGetApp()->LoadIcon(IDI_START));

	if (!bShutdown)
		EnableControlPanel();
}

//===============================================================================================
// FUNCTION: SetProtocol
// PURPOSE:  .
//
bool CDD1550BDlg::SetProtocol()
{
	m_Protocol.pAIBuffers = m_AIBuffers;
	m_Protocol.uAIBuffers = m_AIBuffers.GetCount();
	m_Protocol.pAOBuffers = m_AOBuffers;
	m_Protocol.uAOBuffers = m_AOBuffers.GetCount();

	
	m_Protocol.nNoOf_ANC_ChannelP   = m_DeviceInfo.ucANC_Enable;         // NST Assign Value of Number of ANC Channel from device
	m_Protocol.dMinSequencePeriodUS = m_DeviceInfo.dMinSequencePeriodUS; //NST  Assign Value of Minimum sequence period from Device 6 June12
	m_Protocol.dMaxSequencePeriodUS = m_DeviceInfo.dMaxSequencePeriodUS; //NST  Assign Value of Maximum Sequence Period from Device 6 June12

	//Loop thru protocol disabling ANC
	switch (m_Protocol.nNoOf_ANC_ChannelP)
	{
		case 8:
			break;
		case 1:
			for(UINT i=1;i<DD1550B_MAX_ANC_CHANNELS; i++)
				m_Protocol.anANCEnable[i]= 0;
			break;

		case 4:
			for(UINT i=1;i<8; i = i +2)
				m_Protocol.anANCEnable[i]= 0;
			break;
		case 2:
			m_Protocol.anANCEnable[1]= 0;
			for(UINT i=3;i<8; i++)
				m_Protocol.anANCEnable[i]= 0;
			break;
	}

	if (!m_pDevice->SetProtocol(&m_Protocol))
		return ShowDeviceError();

	if (!m_pDevice->GetProtocol(&m_Protocol))
		return ShowDeviceError();


	
	EnableControlPanel();
	return true;
}

//===============================================================================================
// FUNCTION: ShowPeriod
// PURPOSE:  .
//
void CDD1550BDlg::ShowPeriod(double dUS)
{
	MEMBERASSERT();

	char sz[32];
	MDC_sprintf(sz, ELEMENTS_IN(sz), "%.2f us", dUS);
	SetDlgItemText(IDC_PERIOD, sz);
}

//===============================================================================================
// FUNCTION: Start
// PURPOSE:  .
//
bool CDD1550BDlg::Start()
{
	SetDlgItemText(IDC_STATUS, "");

	if (!m_pDevice)
		return false;

	Stop();
	OnBnClickedOutputs();

	m_NextWave = m_OutPeriod;
	m_NextOut  = m_AOBuffers;
	if (m_uAOSeqLen)
		TopUpOutput(m_AOBuffers.GetTotalSamples()/m_uAOSeqLen);

	m_uLastTime    = 0;
	m_uAISequences = 0;
	m_uAOSequences = 0;
	m_NextIn       = m_AIBuffers;
	m_bEvent       = false;
	m_Synch.clear();

	if (!SetProtocol())
		return false;

	TRACE1("Period = %g us\n", m_Protocol.dSequencePeriodUS);

	m_Stats.SetBuffer(m_AIBuffers, 0, m_uAISeqLen);
	SCOPE_SetDataBuffers( m_hwndScope, m_AIBuffers, NULL, NULL, NULL, m_AIBuffers.GetCount() );
	// SCOPE_FullScaleXAxis( m_hwndScope );
	SCOPE_ClearDisplay( m_hwndScope );
	::UpdateWindow(m_hwndScope);

	if (!m_AcqTimer.Start(m_hWnd, c_uTimerID, c_uTimerPeriod, TRUE))
	{
		MessageBox("No Windows timers available!", "Error");
		return false;
	}
	////////log deleted /////////////////
	//  remove("C:\\TestBed\\WriteFIFOlogFile.txt");
	//  remove("C:\\TestBed\\ReadFIFOlogFile.txt");

	if (!m_pDevice->StartAcquisition())
		return ShowDeviceError();

	ShowPeriod(m_Protocol.dSequencePeriodUS);
	m_Start.SetIcon(AfxGetApp()->LoadIcon(IDI_STOP));
	EnableControlPanel();
	return true;
}

//===============================================================================================
// FUNCTION: FormatCount
// PURPOSE:  .
//
static void FormatCount(LONGLONG uCount, LPSTR psz, UINT uLen)
{
	if (uCount < 100000)
		MDC_sprintf(psz, uLen, "%u", UINT(uCount));
	else if (uCount < 100000000)
		MDC_sprintf(psz, uLen, "%.2f K", double(uCount)/1000);
	else
		MDC_sprintf(psz, uLen, "%.2f M", double(uCount)/1000000);
}

//===============================================================================================
// FUNCTION: DetectEvents
// PURPOSE:  .
//
void CDD1550BDlg::DetectEvents(UINT uSequences)
{
	MEMBERASSERT();

	UINT uRemaining = uSequences * m_uAISeqLen;
	if (m_uAISequences > 0xFFFFFFFF)
		return;

	while (uRemaining)
	{
		UINT uChunk = min(uRemaining, m_NextIn.Remaining());
		const ADC_VALUE *pData = m_NextIn.GetPtr();
		if ((m_Protocol.uFlags & DD1550B_FLAG_TAG_BIT0)==0)
			pData += m_Protocol.uAIChannels;

		UINT uCount = 0;
		if (m_bEvent)
		{
			for (uCount=0; uCount<uChunk; uCount+=m_uAISeqLen)
			{
				if ((pData[uCount] & 1)==0)
				{
					m_bEvent = false;
					break;
				}
			}
			ASSERT(m_Synch.size() > 0);
			m_Synch.back().dwLength += uCount;
		}
		else
		{
			for (uCount=0; uCount<uChunk; uCount+=m_uAISeqLen)
			{
				if (pData[uCount] & 1)
				{
					m_bEvent = true;

					SC_SYNCH Synch;
					Synch.dwStart = DWORD(m_uAISequences*m_uAISeqLen - uRemaining + uCount);
					m_Synch.push_back(Synch);
					break;
				}
			}
		}
		uRemaining -= uCount;
		m_NextIn   += uCount;
	}

#if 0 // def _DEBUG   // DEBUG DEBUG DEBUG
	if (m_Protocol.bDIEnable)
	{
		MDC_DataIterator I = m_NextIn;
		I -= m_uAISeqLen;
		const ADC_VALUE *pData = I.GetPtr();
		TRACE2("DI = %d (0x%04X)\n", pData[m_uAISeqLen-1], pData[m_uAISeqLen-1]);
	}
#endif          // DEBUG DEBUG DEBUG
}

//===============================================================================================
// FUNCTION: OnTimer
// PURPOSE:  .
//
void CDD1550BDlg::OnTimer(UINT_PTR uIDEvent)
{
	CDD1550BDlg_BASE::OnTimer(uIDEvent);

	int nLastError = 0;
	LONGLONG uAISequences = 0, uAOSequences = 0;
	if (!m_pDevice->GetAOPosition(uAOSequences))
	{
		nLastError = m_pDevice->GetLastError();
		Stop();
		ShowDeviceError(nLastError);
		return;
	}

	if (!m_pDevice->GetAIPosition(uAISequences))
	{
		nLastError = m_pDevice->GetLastError();
		Stop();
		ShowDeviceError(nLastError);
	}
	else if (uAISequences)
	{
		UINT uNewAISequences = UINT(uAISequences - m_uAISequences);
		UINT uNewAOSequences = UINT(uAOSequences - m_uAOSequences);
		m_uAISequences     = uAISequences;
		m_uAOSequences     = uAOSequences;

		if (m_uAOSeqLen)
			TopUpOutput(uNewAOSequences);

		if (m_Protocol.bDIEnable || (m_Protocol.uFlags & DD1550B_FLAG_TAG_BIT0))
		{
			DetectEvents(uNewAISequences);
			SCOPE_DrawEventData( m_hwndScope, uNewAISequences, &m_Synch[0], m_Synch.size() );
		}
		else
			SCOPE_DrawData( m_hwndScope, uNewAISequences );

		if (m_Protocol.bMonitorAI)
			m_Stats.NewData(uNewAISequences);
	}

	UINT uTime = m_AcqTimer.GetTimeMS()/1000;
	if ((uTime != m_uLastTime) || nLastError)
	{
		char szIn[64]="", szOut[64]="";
		FormatCount(m_uAISequences, szIn, ELEMENTS_IN(szIn));
		FormatCount(m_uAOSequences, szOut, ELEMENTS_IN(szOut));

		char sz[256] = "";

		if (m_Protocol.bMonitorAI)
		{
			ChannelStats Stats;
			m_Stats.GetResults(Stats);
			MDC_sprintf(sz, ELEMENTS_IN(sz), "%u s\nin:\n%s\n\nStdDev: %.2f\nRange: %d", 
				uTime, szIn, Stats.dStdDev, Stats.nMax-Stats.nMin);
			m_Stats.Clear();
		}
		else
			MDC_sprintf(sz, ELEMENTS_IN(sz), "%u s\n\nin:\n%s\n\nout:\n%s", uTime, szIn, szOut);

		SetDlgItemText(IDC_STATUS, sz);
		m_uLastTime = uTime;
	}

	m_AcqTimer.Acknowledge();
}

//===============================================================================================
// FUNCTION: OnStart
// PURPOSE:  .
//
void CDD1550BDlg::OnStart() 
{
	if(m_pDevice== NULL)  // NST 
	{	
		ShowDeviceError(DD1550B_ERROR_NOTPRESENT);
		return ;
	}

	if (m_AcqTimer.IsRunning())
		Stop();
	else
		Start();
}

//===============================================================================================
// FUNCTION: FixLayout
// PURPOSE:  .
//
void CDD1550BDlg::FixLayout() 
{
	CRect rClient, rScope, rMsg;
	GetClientRect(rClient);
	ClientToScreen(rClient);

	::GetWindowRect(m_hwndScope, rScope);
	rScope.BottomRight() = rClient.BottomRight();
	MDC_SetWindowRect(m_hwndScope, rScope);

	CWnd *pMsg = GetDlgItem(IDC_MSG);
	if (pMsg)
	{
		pMsg->GetWindowRect(rMsg);
		rMsg.OffsetRect(0, rClient.bottom - rMsg.bottom);
		MDC_SetWindowRect(pMsg->m_hWnd, rMsg);
	}
}

//===============================================================================================
// FUNCTION: OnSize
// PURPOSE:  .
//
void CDD1550BDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDD1550BDlg_BASE::OnSize(nType, cx, cy);
	FixLayout();
}

//===============================================================================================
// FUNCTION: OnPaint
// PURPOSE:  .
//
// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CDD1550BDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}

	if (!m_bInit)
	{
		PostMessage(WM_OPEN_DEVICE);
		m_bInit = true;
	}
}

//===============================================================================================
// FUNCTION: OnQueryDragIcon
// PURPOSE:  .
//
// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CDD1550BDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

//===============================================================================================
// FUNCTION: OnDestroy
// PURPOSE:  .
//
void CDD1550BDlg::OnDestroy() 
{
	SCOPE_SerializeToINI( m_hwndScope, "Scope", "DD1550Dlg", NULL);

	CloseDevice();
	CDD1550BDlg_BASE::OnDestroy();
}

void CDD1550BDlg::OnBnClickedProtocol()
{
	CProtocolDlg Dlg(this, &m_Protocol);
	if (Dlg.DoModal() == IDCANCEL)
		return;
	m_Protocol.Save();
	SetupScope();
	SetPeriod(m_Protocol.dSequencePeriodUS);
    
	if(m_pDevice== NULL)  // NST 
	{	
		ShowDeviceError(DD1550B_ERROR_NOTPRESENT);
		return ;
	}
	VERIFY(SetProtocol());

	UpdateData(false);
	OnBnClickedOutputs();
}

//===============================================================================================
// FUNCTION: ReadInputs
// PURPOSE:  .
//
void CDD1550BDlg::ReadInputs()
{
	MEMBERASSERT();
	if (m_AcqTimer.IsRunning())
	{
		// MessageBeep(MB_CANCEL);
		return;
	}

	char szInputs[1024] = "", sz[64] = "";
	for (UINT i=0; i<m_Protocol.uAIChannels; i++)
	{
		short nValue = 0;
		if (!m_pDevice->GetAIValue(m_Protocol.anAIChannels[i], nValue))
		{
			ShowDeviceError();
			break;
		}

		MDC_sprintf(sz, ELEMENTS_IN(sz), "AI%d = %d\n", m_Protocol.anAIChannels[i], nValue);
		MDC_strncatz(szInputs, sz);
	}

	if (m_Protocol.bDIEnable)
	{
		WORD wValue = 0;
		if (!m_pDevice->GetDIValue(wValue))
			ShowDeviceError();

		MDC_sprintf(sz, ELEMENTS_IN(sz), "DI = 0x%02X\n", wValue);
		MDC_strncatz(szInputs, sz);
	}
	SetDlgItemText(IDC_STATUS, szInputs);
}

void CDD1550BDlg::OnBnClickedTestInputs()
{
	m_TestInputsDlg.ShowWindow(IsDlgButtonChecked(IDC_TEST_INPUTS) ? SW_SHOWNORMAL : SW_HIDE);
}



void CDD1550BDlg::OnBnClickedOutputs()
{
	if (!UpdateData())
		return;


	if(m_pDevice== NULL)  // NST 
	{	
		ShowDeviceError(DD1550B_ERROR_NOTPRESENT);
		return ;
	}


	BOOL bAcquiring = m_AcqTimer.IsRunning();

	UINT uAOMask = m_Protocol.GetAOMask();
	bool bOK = true;
	for (UINT i=0; i<ELEMENTS_IN(m_Protocol.adAOHolding); i++)
	{
		// If this channel is being driven, do not try to set it.
		if (bAcquiring && (uAOMask & (1<<i)))
			continue;

		DAC_VALUE nDAC = DD1550B_VoltsToDAC(m_CalData, i, m_Protocol.adAOHolding[i]);
		//m_Protocol.adAOHolding[i] = DD1550B_DACtoVolts(m_CalData, i, nDAC);  // NST Comment as per santosha we are not require assign calculated value

		if (!m_pDevice->SetAOValue(i, nDAC))
			bOK = false;
	}

	// Only try to set digital outs if not acquiring or digital outs not driven during acquisition.
	if (!bAcquiring || !m_Protocol.bDOEnable)
	{
		if (!m_pDevice->SetDOValue(m_Protocol.wDOHolding))   
			bOK = false;
	}

	UpdateData(false);
	if (!bOK)
		ShowDeviceError();
}

//===============================================================================================
// FUNCTION: void OnBnClickedDO
// PURPOSE:  .
//
void CDD1550BDlg::OnBnClickedDO(UINT uID)
{
	MEMBERASSERT();
	OnBnClickedOutputs();
}

void CDD1550BDlg::OnBnClickedRotate()
{
	
	if(m_pDevice== NULL)  // NST 
	{	
		ShowDeviceError(DD1550B_ERROR_NOTPRESENT);
		return ;
	}

	OnBnClickedOutputs();

	UINT uHolding = m_Protocol.wDOHolding;
	if (GetKeyState(VK_SHIFT) < 0)
	{
		UINT uLoBit = uHolding&1;
		uHolding >>= 1;
		if (uLoBit)
			uHolding |= 0x8000;
	}
	else
	{
		 uHolding <<= 1;
		
		if((m_Protocol.uFlags & (1<<3)) & DD1550B_FLAG_SCOPE_OUT )
		{
			if(LOWORD( m_Protocol.wDOHolding)& (1<<14))
			{
             uHolding = LOWORD(uHolding)|1;
			}

		}		
				
		if (HIWORD(uHolding))
			uHolding = LOWORD(uHolding) | 1;
	}

	
	m_Protocol.wDOHolding = LOWORD(uHolding);
	UpdateData(false);

	if (!m_pDevice->SetDOValue(m_Protocol.wDOHolding))
		ShowDeviceError();
}

//===============================================================================================
// FUNCTION: EnableControlPanel
// PURPOSE:  .
//
void CDD1550BDlg::EnableControlPanel()
{
	MEMBERASSERT();
	if (!m_hWnd)
		return;

	bool bStopped = !m_pDevice->IsAcquiring();

	UINT uAOMask = m_Protocol.GetAOMask();
	for (UINT i=0; i<ELEMENTS_IN(m_Protocol.adAOHolding); i++)
		GetDlgItem(IDC_AO0+i)->EnableWindow(bStopped || ((uAOMask & (1<<i)) == 0));

	for (UINT i=0; i<DD1550B_MAX_DO_CHANNELS; i++) // NST Change maximum Digital output channel to 16 from 8  for req 2 
		GetDlgItem(IDC_DO0+i)->EnableWindow(bStopped || !m_Protocol.bDOEnable);

	m_EditProtocol.EnableWindow(bStopped);
	m_TestInputsDlg.Update();
}

//===============================================================================================
// FUNCTION: OnAbout
// PURPOSE:  .
//
void CDD1550BDlg::OnAbout() 
{
	CAboutDlg Dlg(this, m_pDevice);
	Dlg.DoModal();

	if (Dlg.SerialChanged())
	{
		char sz[128] = "";
		m_pDevice->GetDeviceInfo(&m_DeviceInfo);
		MDC_sprintf(sz, ELEMENTS_IN(sz), "Serial\n#%d", m_DeviceInfo.uSerialNumber);
		SetDlgItemText(IDC_SERIAL, sz);
	}
}

//===============================================================================================
// FUNCTION: OnClear
// PURPOSE:  .
//
void CDD1550BDlg::OnClear() 
{
	SCOPE_ClearDisplay( m_hwndScope );
}

//===============================================================================================
// FUNCTION: OnCopy
// PURPOSE:  .
//
void CDD1550BDlg::OnCopy() 
{
	SCOPE_CopyToClipboard( m_hwndScope );
}

//===============================================================================================
// FUNCTION: OnScopeMessage
// PURPOSE:  .
//
LRESULT CDD1550BDlg::OnScopeMessage(WPARAM wParam, LPARAM lParam)
{
	MEMBERASSERT();
	switch (wParam)
	{
	case SCN_PROPERTIESCHANGED:
		break;

	case SCN_DOUBLECLICK:
	case SCN_CLICK_RMOUSE:
		break;
		// return (GetKeyState(VK_SHIFT) < 0) ? SCN_NOT_PROCESSED : SCN_PROCESSED;
	}

	return SCN_NOT_PROCESSED;
}

//===============================================================================================
// FUNCTION: OnUIUpdate
// PURPOSE:  .
//
LRESULT CDD1550BDlg::OnUIUpdate(WPARAM wParam, LPARAM lParam)
{
	MEMBERASSERT();

	CheckDlgButton(IDC_TEST_INPUTS, m_TestInputsDlg.IsWindowVisible());
	return 0;
}

//===============================================================================================
// FUNCTION: SetPeriod
// PURPOSE:  .
//
void CDD1550BDlg::SetPeriod(double dPeriodUS)
{
	dPeriodUS = max(dPeriodUS, m_DeviceInfo.dMinSequencePeriodUS);
	dPeriodUS = min(dPeriodUS, m_DeviceInfo.dMaxSequencePeriodUS);

	int nQuanta = Round(dPeriodUS / m_DeviceInfo.dSequenceQuantaUS);
	m_Protocol.dSequencePeriodUS = nQuanta * m_DeviceInfo.dSequenceQuantaUS;

	ShowPeriod(m_Protocol.dSequencePeriodUS);
}

//===============================================================================================
// FUNCTION: OnCommand
// PURPOSE:  .
//
BOOL CDD1550BDlg::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if (LOWORD(wParam) == IDC_PERIOD_SPINNER)
	{
		bool bUp = (HIWORD(wParam)==MCC_SPINNER_NOTIFY_UP);

		if ((Round(m_Protocol.dSequencePeriodUS) == Round(m_DeviceInfo.dMaxSequencePeriodUS) && bUp) ||
			(Round(m_Protocol.dSequencePeriodUS) == Round(m_DeviceInfo.dMinSequencePeriodUS) && !bUp))
		{
			MessageBeep(UINT_MAX);
		}
		else
		{
			BOOL bRunning = m_AcqTimer.IsRunning();

			if (bRunning)
				Stop();

			SetPeriod(m_Protocol.dSequencePeriodUS * (bUp ? 2.0 : (1.0/2.0)));
			SetupScope();

			if (bRunning)
				Start();
		}
	}
	return CDD1550BDlg_BASE::OnCommand(wParam, lParam);
}

void CDD1550BDlg::OnBnClickedOpenDevice()
{
	m_uSerial = UINT(-1);
	PostMessage(WM_OPEN_DEVICE);
}

//===============================================================================================
// FUNCTION: ShowSuccess
// PURPOSE:  .
//
void CDD1550BDlg::ShowSuccess(UINT uID, bool bSuccess)
{
	MEMBERASSERT();
	UINT uIcon = bSuccess ? IDI_CHECK : IDI_CROSS;

	CStatic *pStatic = (CStatic *)GetDlgItem(uID);

	pStatic->SetIcon(AfxGetApp()->LoadIcon(uIcon));
	pStatic->ShowWindow(SW_SHOWNORMAL);
}



/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog

CAboutDlg::CAboutDlg(CWnd* pParent, DD1550B_Device *pDevice)
: CDialog(CAboutDlg::IDD, pParent)
{
	m_pDevice        = pDevice;
	m_bSerialChanged = false;

	if (m_pDevice)
		VERIFY(m_pDevice->GetDeviceInfo(&m_Info));

	//{{AFX_DATA_INIT(CAboutDlg)
	// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_PRODUCT, m_Product);
	DDX_Control(pDX, IDC_USB1, m_USB1);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	ON_BN_CLICKED(IDC_SET_SERIAL, OnBnClickedSetSerial)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg message handlers

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// Get the product version and the internal name
	{
		MDC_Version Version(AfxGetResourceHandle());
		ASSERT(Version.IsValid());

		char szName[32] = "", szVer[32]="", szBuf[256] = "";
		Version.GetString("InternalName", szName, sizeof(szName));

		strcpy(szName, "DD1550B.exe");

		DWORD dwVersion = 0;
		Version.GetFileVersion(&dwVersion);

		MDC_sprintf(szBuf, ELEMENTS_IN(szBuf), "%s %d.%d", szName, int(HIWORD(dwVersion)), int(LOWORD(dwVersion)));
		SetDlgItemText(IDC_PRODUCT, szBuf);

		LOGFONT LogFont   = { 0 };
		LogFont.lfHeight  = 200;
		LogFont.lfWeight  = FW_BOLD;
		LogFont.lfCharSet = ANSI_CHARSET;
		MDC_strncpyz(LogFont.lfFaceName, "Arial");
		if (m_Font.CreatePointFontIndirect(&LogFont))
			m_Product.SetFont(&m_Font, FALSE);

		Version.GetString("FileVersion", szVer, sizeof(szVer));
		MDC_sprintf(szBuf, ELEMENTS_IN(szBuf), "%s\n%s", szName, szVer);
		SetDlgItemText(IDC_APP, szBuf);
	}

	// Get the driver version and the internal name
	{
		MDC_Version Version(MDC_ModulePath("AxDD1550B.dll"));
		ASSERT(Version.IsValid());

		char szName[32] = "", szVer[32]="", szBuf[256] = "";
		Version.GetString("InternalName", szName, sizeof(szName));

		Version.GetString("FileVersion", szVer, sizeof(szVer));
		MDC_sprintf(szBuf, ELEMENTS_IN(szBuf), "%s\n%s", szName, szVer);
		SetDlgItemText(IDC_DRIVER, szBuf);
	}

	SetDlgItemText(IDC_NAME,     m_Info.szName);
	SetDlgItemText(IDC_FIRMWARE, m_Info.szFirmwareVersion);
	SetDlgItemInt(IDC_SERIAL,    m_Info.uSerialNumber);
	SetDlgItemText(IDC_FGGA_VER, m_Info.szFpgaVersion); // NST show Firmware version  21.08.12



	m_USB1.ShowWindow(m_Info.bUSB1 ? SW_SHOWNORMAL : SW_HIDE);

	GetDlgItem(IDC_SET_SERIAL)->EnableWindow(m_pDevice!=0);
	GetDlgItem(IDC_SERIAL)->EnableWindow(m_pDevice!=0);

	return TRUE;  // return TRUE unless you set the focus to a control
}

void CAboutDlg::OnBnClickedSetSerial()
{
	UINT uSerialNumber = GetDlgItemInt(IDC_SERIAL);

	VERIFY(m_pDevice->SetSerialNumber(uSerialNumber));

	m_bSerialChanged = true;
}


