//***********************************************************************************************
//
//    Copyright (c) 2005 Molecular Devices.
//    All rights reserved.
//
//***********************************************************************************************
// MODULE:  Protocol.CPP
// PURPOSE: 
// AUTHOR:  BHI  Feb 2005
//

#include "stdafx.h"
#include "Protocol.hpp"

DECLARE_STR(Protocol);
DECLARE_STR(SequenceUS);
DECLARE_STR(Flags);
DECLARE_STR(AIChannels);
DECLARE_STR(AOChannels);
DECLARE_STR(DIEnable);
DECLARE_STR(DOEnable);
DECLARE_STR(SaveAI);
DECLARE_STR(SaveAO);
DECLARE_STR(DOHolding);
DECLARE_STR(MonitorAI);
DECLARE_STR(TerminalCount);
DECLARE_STR(OutPeriodMS);
DECLARE_STR(OutMaxV);
DECLARE_STR(OutMinV);
DECLARE_STR(ANCEnable);
DECLARE_STR(NumberOfANCChannel);
DECLARE_STR(ScopeOutAIChannel);
DECLARE_STR(ScopeOutThreshold);
DECLARE_STR(ScopeOutHysteresis);
DECLARE_STR(ScopeOutPolarity);
DECLARE_STR(NumOfAvgCycle);

const char c_szApp[] = "DD1550B";

void Protocol::Init()
{
	dSequencePeriodUS = 10;
	uFlags            = 0;              // Boolean flags that control options.

	uAIChannels       = 1;
	anAIChannels[0]   = 0;

	uAOChannels       = 1;
	anAOChannels[0]   = 0;

	memset(adAOHolding, 0, sizeof(adAOHolding));
	wDOHolding        = 0;

	bMonitorAI        = false;
	bScopeOutPolarity = true;

	memset(anAOWaveType, 0, sizeof(anAOWaveType));
	memset(anDOWaveType, 0, sizeof(anDOWaveType));
	dOutPeriodMS      = 1000;
	dOutMaxV          = 10;
	dOutMinV          = -10;
	anANCEnable[0]	 = 0;
	nNoOf_ANC_ChannelP = 0; 
	dMinSequencePeriodUS=2;  // NST Minimum sequence period 500 KHz.
	dMaxSequencePeriodUS= 1000000;
	uNoOfAvgCycle       = 0;
}

UINT Protocol::GetAIMask() const
{
	UINT uAIMask = 0;
	for (UINT i=0; i<uAIChannels; i++)
		uAIMask |= (1<<anAIChannels[i]);
	return uAIMask;
}

UINT Protocol::GetAOMask() const
{
	UINT uAOMask = 0;
	for (UINT i=0; i<uAOChannels; i++)
		uAOMask |= (1<<anAOChannels[i]);
	return uAOMask;
}

void Protocol::Load(LPCSTR pszFilename)
{
	Init();
	BOOL bINI = false; 
	if (!pszFilename)
	{
		pszFilename = c_szApp;
		bINI = true;
	}
	BOOL bSave = MDC_UseINIFile(bINI);

	MDC_GetProfileDouble(USE_STR(Protocol), USE_STR(SequenceUS), dSequencePeriodUS, &dSequencePeriodUS, c_szApp);
	MDC_GetProfileUINT(USE_STR(Protocol), USE_STR(Flags), uFlags, &uFlags, c_szApp);

	UINT uAIMask = 1;
	uAIChannels  = 0;
	MDC_GetProfileUINT(USE_STR(Protocol), USE_STR(AIChannels), uAIMask, &uAIMask, c_szApp);
	for (UINT i=0; i<DD1550B_MAX_AI_CHANNELS; i++)
		if (uAIMask & (1<<i))
			anAIChannels[uAIChannels++] = i;
	if (uAIChannels==0)
		anAIChannels[uAIChannels++] = 0;

	UINT uAOMask = 1;
	uAOChannels  = 0;
	MDC_GetProfileUINT(USE_STR(Protocol), USE_STR(AOChannels), uAOMask, &uAOMask, c_szApp);
	for (UINT i=0; i<DD1550B_MAX_AO_CHANNELS; i++)
		if (uAOMask & (1<<i))
			anAOChannels[uAOChannels++] = i;

	MDC_GetProfileBool(USE_STR(Protocol), USE_STR(DIEnable), bDIEnable, &bDIEnable, c_szApp);
	MDC_GetProfileBool(USE_STR(Protocol), USE_STR(DOEnable), bDOEnable, &bDOEnable, c_szApp);

	MDC_GetProfileBool(USE_STR(Protocol), USE_STR(SaveAI), bSaveAI, &bSaveAI, c_szApp);
	MDC_GetProfileBool(USE_STR(Protocol), USE_STR(SaveAO), bSaveAO, &bSaveAO, c_szApp);
	MDC_GetProfileUINT(USE_STR(Protocol), USE_STR(TerminalCount), uTerminalCount, &uTerminalCount, c_szApp);

	for (UINT i=0; i<DD1550B_MAX_AO_CHANNELS; i++)
	{
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "AOVolts%d", i);
		MDC_GetProfileDouble(USE_STR(Protocol), sz, adAOHolding[i], adAOHolding+i, c_szApp);
	}
	MDC_GetProfileWORD(USE_STR(Protocol), USE_STR(DOHolding), wDOHolding, &wDOHolding, c_szApp);

	MDC_GetProfileBool(USE_STR(Protocol), USE_STR(MonitorAI), bMonitorAI, &bMonitorAI, c_szApp);


	for (UINT i=0; i<DD1550B_MAX_AO_CHANNELS; i++)
	{
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "AOWave%d", i);
		MDC_GetProfileInt(USE_STR(Protocol), sz, anAOWaveType[i], anAOWaveType+i, c_szApp);
	}
	for (UINT i=0; i<DD1550B_MAX_DO_CHANNELS; i++)
	{
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "DOWave%d", i);
		MDC_GetProfileInt(USE_STR(Protocol), sz, anDOWaveType[i], anDOWaveType+i, c_szApp);
	}
	MDC_GetProfileDouble(USE_STR(Protocol), USE_STR(OutPeriodMS), dOutPeriodMS, &dOutPeriodMS, c_szApp);
	MDC_GetProfileDouble(USE_STR(Protocol), USE_STR(OutMaxV), dOutMaxV, &dOutMaxV, c_szApp);
	MDC_GetProfileDouble(USE_STR(Protocol), USE_STR(OutMinV), dOutMinV, &dOutMinV, c_szApp);

	for(UINT i=0;i<DD1550B_MAX_ANC_CHANNELS;i++)
	{	
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "ANCEnable%d", i);
		MDC_GetProfileInt(USE_STR(Protocol), sz, anANCEnable[i], anANCEnable+i, c_szApp); // NST Load ANC Enable channel value  for Req 4
	}

    MDC_GetProfileInt(USE_STR(Protocol), USE_STR(NumOfAvgCycle), uNoOfAvgCycle, &uNoOfAvgCycle, c_szApp); // NST Load Number of Average Cycle

	MDC_GetProfileInt(USE_STR(Protocol), USE_STR(NumberOfANCChannel), nNoOf_ANC_ChannelP, &nNoOf_ANC_ChannelP, c_szApp); // NST Load Number of ANC Channels value  for Req 4
	
	MDC_GetProfileInt(USE_STR(Protocol), USE_STR(ScopeOutAIChannel), nScopeOutAIChannel, &nScopeOutAIChannel, c_szApp);
	MDC_GetProfileShort(USE_STR(Protocol), USE_STR(ScopeOutThreshold), nScopeOutThreshold, &nScopeOutThreshold, c_szApp);
	MDC_GetProfileShort(USE_STR(Protocol), USE_STR(ScopeOutHysteresis), nScopeOutHysteresis, &nScopeOutHysteresis, c_szApp);
	MDC_GetProfileBool(USE_STR(Protocol), USE_STR(ScopeOutPolarity), bScopeOutPolarity, &bScopeOutPolarity, c_szApp);

	
	
	MDC_UseINIFile(bSave);
}

void Protocol::Save(LPCSTR pszFilename) const
{
	BOOL bINI = false; 
	if (!pszFilename)
	{
		pszFilename = c_szApp;
		bINI = true;
	}
	BOOL bSave = MDC_UseINIFile(bINI);

	MDC_WriteProfileDouble(USE_STR(Protocol), USE_STR(SequenceUS), dSequencePeriodUS, c_szApp);
	MDC_WriteProfileUINT(USE_STR(Protocol), USE_STR(Flags), uFlags, c_szApp);

	UINT uAIMask = max(GetAIMask(), 1);
	MDC_WriteProfileUINT(USE_STR(Protocol), USE_STR(AIChannels), uAIMask, c_szApp);

	UINT uAOMask = GetAOMask();
	MDC_WriteProfileUINT(USE_STR(Protocol), USE_STR(AOChannels), uAOMask, c_szApp);

	MDC_WriteProfileBOOL(USE_STR(Protocol), USE_STR(DIEnable), bDIEnable, c_szApp);
	MDC_WriteProfileBOOL(USE_STR(Protocol), USE_STR(DOEnable), bDOEnable, c_szApp);

	MDC_WriteProfileBool(USE_STR(Protocol), USE_STR(SaveAI), bSaveAI, c_szApp);
	MDC_WriteProfileBool(USE_STR(Protocol), USE_STR(SaveAO), bSaveAO, c_szApp);
	MDC_WriteProfileUINT(USE_STR(Protocol), USE_STR(TerminalCount), uTerminalCount, c_szApp);

	for (UINT i=0; i<DD1550B_MAX_AO_CHANNELS; i++)
	{
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "AOVolts%d", i);
		MDC_WriteProfileDouble(USE_STR(Protocol), sz, adAOHolding[i], c_szApp);
	}
	MDC_WriteProfileWORD(USE_STR(Protocol), USE_STR(DOHolding), wDOHolding, c_szApp);

	MDC_WriteProfileBool(USE_STR(Protocol), USE_STR(MonitorAI), bMonitorAI, c_szApp);

	for (UINT i=0; i<DD1550B_MAX_AO_CHANNELS; i++)
	{
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "AOWave%d", i);
		MDC_WriteProfileInt(USE_STR(Protocol), sz, anAOWaveType[i], c_szApp);
	}
	for (UINT i=0; i<DD1550B_MAX_DO_CHANNELS; i++)
	{
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "DOWave%d", i);
		MDC_WriteProfileInt(USE_STR(Protocol), sz, anDOWaveType[i], c_szApp);
	}
	MDC_WriteProfileDouble(USE_STR(Protocol), USE_STR(OutPeriodMS), dOutPeriodMS, c_szApp);
	MDC_WriteProfileDouble(USE_STR(Protocol), USE_STR(OutMaxV), dOutMaxV, c_szApp);
	MDC_WriteProfileDouble(USE_STR(Protocol), USE_STR(OutMinV), dOutMinV, c_szApp);

	for(UINT i=0;i<DD1550B_MAX_ANC_CHANNELS;i++)
	{
		char sz[32] = "";
		MDC_sprintf(sz, ELEMENTS_IN(sz), "ANCEnable%d", i);
		MDC_WriteProfileInt(USE_STR(Protocol), sz, anANCEnable[i], c_szApp); // NST write ANC Enable channel value  for Req 4
	}

	MDC_WriteProfileInt(USE_STR(Protocol), USE_STR(NumOfAvgCycle), uNoOfAvgCycle, c_szApp);  // NST Write Number of Average Cycle

	MDC_WriteProfileInt(USE_STR(Protocol), USE_STR(NumberOfANCChannel), nNoOf_ANC_ChannelP, c_szApp); // NST write Number of ANC Channels value  for Req 4
	
	MDC_WriteProfileInt(USE_STR(Protocol), USE_STR(ScopeOutAIChannel), nScopeOutAIChannel, c_szApp); 
	MDC_WriteProfileShort(USE_STR(Protocol), USE_STR(ScopeOutThreshold), nScopeOutThreshold, c_szApp); 
	MDC_WriteProfileShort(USE_STR(Protocol), USE_STR(ScopeOutHysteresis), nScopeOutHysteresis, c_szApp);
	MDC_WriteProfileBOOL(USE_STR(Protocol), USE_STR(ScopeOutPolarity), bScopeOutPolarity, c_szApp);
	
	
	MDC_UseINIFile(bSave);
}
